"Change Set:		OSProcessV4-3-5-dtl
Date:			22 December 2007
Author:			David T. Lewis

OSProcess provides access to the external operating system from Squeak. Plugins are provided for Unix, Linux, and Windows systems. OSProcess can be loaded on other platforms as well, and placeholder classes are provided in the OSProcess hierarchy for other operating systems (Mac, OS/2, RiscOS), although support for these systems is not yet implemented.

Prerequisites:
 - Your VM should have an OSProcess plugin available. If it does not, load the OSProcessPlugin package and build the plugin with VMMaker.
 - If you use X11 (most Unix/Linux systems), your VM should have an XDisplayControlPlugin available. If it does not, load XDisplayControlPlugin and build the plugin with VMMaker.

See the separate CommandShell change set for a Unix shell window that uses OSProcess.

You can:
 - Open a inspector on the operating system process in which Squeak is running (OSProcess thisOSProcess).
 - Read or write on the stdin, stdout, and stderr streams (Unix only for this release).
 - Access the command line, and get or set environment variables (Windows: get environment only).
 - Fork and exec external programs, with control of the command line, environment, and stdin/stdout/stderr (Windows: run an external program).
 - Open an inspector on a child operating system process created from Squeak, with run state and exit status of the child updated asynchronously.
 - Run an external command:
       OSProcess command: 'xeyes' (Unix)
       OSProcess command: 'sol' (Windows)
 - Connect directly to input and outputs of external commands. See the separate CommandShell change set for a useful implementation, supporting commands like:
       (PipeableOSProcess command: 'ls -l *') output
 - Create pipes and connect them to external processes (see the CommandShell package).
 - Fork an exact clone of the current running Squeak image (no image file) with parent and child Squeak processes proceeding from the same point in memory (UnixProcess forkSqueak).
 - Fork a Squeak using the disk based image (UnixProcess squeak).
 - Start a headless Squeak from the running Squeak image, giving it an expression to evaluate:
       (UnixProcess forkHeadlessSqueakAndDoThenQuit:
           [UnixProcess command: 'echo hello world'])
 - Tell the running image to go headless (UnixProcess decapitate), and reconnected the image with a window (UnixProcess recapitate).
-  Move the display to another computer on the network:
       (UnixProcess displayOnXServer: 'someOtherComputer')
 - Restart the VM on the fly (UnixProcess restartVirtualMachine).
 - Rebuild the VM, and restart Squeak with the new VM if the build was successful (UnixProcess makeVM).
 - Start your Swiki in a headless background process:
       (UnixProcess startSwiki: 'myswiki' onPort: 8081 loggingTo: 'log.txt')

Known bugs:
- On OS X, display management methods (e.g. #recapitate) and methods
  that require them (e.g. #forkSqueak) will not work. This is because
  an equivalent to XDisplayControlPlugin is not yet available for OS X.
- Stability and functionality of OSProcess on OS X is uncertain, as I do
  not have a Mac system available for testing (thanks to all who have
  helped testing via email).
- On Windows, OSPipe is implemented, but an OSPipe cannot yet be set
  for nonBlocking reads or writes. Reading from an OSPipe with no data
  available will block the Squeak VM.
- On Windows, stdin/stdout/stderr access is not yet implemented.
- On Windows, command pipelines for external commands are not yet
  implemented. CommandShell does work, and may be used for launching
  individual Windows programs as well as for the usual CommandShell
  internal commands and internal command pipelines. However, it is
  not yet possible to construct a pipeline of Windows programs.
- Forking a child Squeak, as in UnixProcess>>forkSqueak, results in two
  instances of Squeak which share a single changes file. This has no
  practical impact if one or the other of the Squeak processes exits without
  doing very much, but it would could result in a corrupted changes
  file if both Squeak instances do a lot of changes. Any problems can
  be avoided by having one of the Squeak instances save itself as a
  new version, so that each instance uses its own image and changes files.
- The connectToXDisplay method is probably dangerous from a security
  perspective.

Recent version history:

Version 4.3.5:
- Fix a concurrency problem in BufferedAsyncFileReadStream>>upToEndOfFile.
  This corrects intermittent failures detected in CommandShell test suite.

Version 4.3.4:
- Update reference to #listLoadedModules to work in newer Squeak versions.
- Update some default path strings for use in CommandShell.
- Removed separate version string for AioEventHandler.

Version 4.3.3:
- Provide backward compatibility with older versions of OSProcessPlugin.
- Fix bug in end of file test with older OSPP.

Version 4.3.2:
- Remove unnecessary synchronization semaphore from BufferedAsyncFileReadStream.
- Implement BufferedAsyncFileReadStream>>upTo:
- Fix underscores.
- Changed startup processing to reinitialize current OS process when image
  moves from one platform to another (e.g. image from Windows moved to Unix).
- Check installed OSPP version before calling pthread primitive to avoid error
  message when using older plugin.
- Add AttachableFileStream>>setBlocking and AttacheableFileStream>>setNonBlocking
  convenience methods.
- Better synchronization of buffer in BufferedAsyncFileReadStream.
- Add #killCurrentChildrenAtExit and #killOnVmExit:withSignal to arrange for child
  processes to be terminated on VM exit even if normal shutdown processing does
  not occur.
- Add accessor methods for Unix setsid(), getpgid(), setpgid(), getpgrp(), setpgrp()
  session management functions.
- Add modifications to StandardFileStream to package.
- Lower the priority of event handler process for AioEventHander to prevent input
  buffer being overloaded whens service process cannot keep up with input stream.

Version 4.3:
- Changes to support a fully event-driven CommandShell.
- Add StandardFileStream modifications to package.
- Change from underscores to ':=' for assignments.
- Win32OSProcessPlugin.dll is no longer distributed in this package.

Version 4.2:
- Fix #isAtEndOfFile: and add AttachableFileStream>>atEnd to make use of the fixed method. OSPP 4.2 has a fixed primitive that will be used if available, otherwise OSP works around the old bug.
- Add a nil check to AttachableFileStream>>atEnd.
- Reinstate methods required for backward compabibility with older CommandShell. Add new methods to implement improved #atEnd behavior, and #upToEndOfFile for pipes.

Version 4.1:
- Add buffered async behavior to file streams.
- Update OSPipe for event driven input.
- Update file locking to fix a bug and provide better error handling.
- Add fail blocks to file lock methods (versus e.g. test for nil).

Version 4.0.3:
- Add pthread instance variable to UnixProcess to display the identity of the pthread in which the interpreter is executing.
- File unlocking could fail if Win32 file locking emulation is disabled. This release fixes the problem. The problem can be demonstrated by evaluating 'OSProcessAccessor emulateWin32FileLocking: false' then running the UnixProcessFileLockingTestCase tests (or the UnixProcessWin32FileLockingTestCase and UnixProcessUnixFileLockingTestCase tests that replace them).

Version 4.0.2:
- Changed #saveImageInBackground to use a better time stamped image name.
- Added #saveImageInBackgroundNicely to do a background image save at lower priority.
- Fixed #uniqueNameForSavedImage to use leading zeros in date fields.
- Turn off Win32 file locking by default.
- Add a UnixProcess example for background processing.
- Clean up logic on OSFileLock>>test.
- Add pthread accessor for UnixOSProcessAccessor (requires OSPP 4.0.2).

Version 4.0.1:
- Added #nice: to set scheduling priority of the Squeak VM process.
- Changed #saveImageInBackground to use a better time stamped image name.
- Added #saveImageInBackgroundNicely to do a background image save at lower priority.
- Added #forwardSigUsr1 and #forwardSigUsr2.
- Added methods for restoring original signal handlers, e.g. #restoreSigUsr1.

Version 4.0:
- Added 64-bit support. With OSPP 4.0 or greater, runs on a 64 bit VM, image and/or 64 bit hardware. Backward compatibility is maintained for 32 bit platforms and earler VMMaker distributions.
- Added AIO classes previously distributed as part of the AIO package.
- Updated #allTestResults to tolerate missing plugins, update test for session ID, and provide better console output and results file.
- Fixed bit rot in Win32 classes.
- Added support for OS threads in Win32.
- Added Windows process access methods for primitives in OSPP 4.0.

Version 3.2.3:
- Added support for KCP refactorings, in which a number methods were moved from SystemDictionary to SmalltalkImage. Backward compatibility is maintained for Squeak 3.6 and earlier.
- Changed class categories to work better with Monticello.
- Various minor cleanups and refactorings for file locking support.
- Added OSPipe>>printOn: to avoid a problem that occurs when using the default Stream implementation.
- Updated #removeAllOSProcessAndCommandShellClassesFromSystem to handle new class categories.

Version 3.2.2:
- Update for Squeak 3.7. In particular, recent improvements to TestRunner flushed out some timing problems in OSProcess (especially in sUnit tests for CommandShell) which are now corrected in the child process reaper UnixOSProcessAccessor>>grimReaperProcess.
- Rewrote primFork methods in both OSProcess and OSPP. This version of OSProcess uses the new primitives in OSPP 3.3 but maintains backward compatibility with earlier OSPP versions (at least back to OSPP 3.1.1). The child process creation methods for Unix now make use of the generic UnixProcessAccessor>>forwardSignal: signal forwarding mechanism rather than having the signal handlers coded in the  individual process creation and pipe creation primitives in OSPP.
- If OSPP is compiled with -D_REENTRANT, OSProcess will probably now work on OS X, except for display management which is required for forkSqueak and friends.
- Better platform awareness for OS X.
- Rewrote file lock Win32 compatibility.
- Add class FileRegion to support Win32 file lock compatibility.

Version 3.2.1:
- Added support for Unix file locking
- Generalized the logic for platform selection and added MacOS unix/non-unix conditions
- Changed forkSqueak methods to fail gracefully if display management is not available.
- Removed obsolete method #primForkHeadlessSqueak.
- fixed return value of #flushXDisplay.

Version 3.1:
- Removed OSProcessPlugin from OSProcess for distribution as a separate package.
- Added XDisplayControlPlugin to replace X11 methods in OSProcessPlugin. This plugin is also distributed separately from OSProcess.
- Changed forkSqueak methods to handle X11 connection from the image (using XDisplayControlPlugin) rather than in OSPP. This anables future implementations of forkSqueak on non-X platforms such as Mac OS/X.
- Added event forwarding methods to OSProcess and OSPP for use in future event-driven OSProcess releases.
- Added various bug fixes and updates to bring OSProcess up to date with Squeak 3.6.
"!

